%% 
%
% Nonlinear and stable perturbation-based approximations
% Wouter J. Den Haan and Joris De Wind
%
% Brock - Mirman programs
%
%   Calculates statistics for Table 1
%   in the directory with your Dynare files you have to replace the disp_dr
%   file with the disp_dr file found in this directory
%
%--------------------------------------------------------------------------

%Cleaning
clear all; close all; clc

global kss beta gamma alpha delta gh sigma rho decision_lev_1st options decision_lev

%% Parameters
%--------------------------------------------------------------------------
options = optimset('Display','off','MaxIter',1E5,'TolFun',1E-10,'TolX',1E-10);

%Model parameters
alpha = 0.36;
beta  = 0.99;
gamma = 1;
delta = 1;
rho   = 0.95;
sigma = 0.2;

%Creating parameters.mod using system commands
delete parameters.mod
system(sprintf('echo alpha = %0.14f; >> parameters.mod',alpha));
system(sprintf('echo beta  = %0.14f; >> parameters.mod',beta ));
system(sprintf('echo gamma = %0.14f; >> parameters.mod',gamma));
system(sprintf('echo delta = %0.14f; >> parameters.mod',delta));
system(sprintf('echo rho   = %0.14f; >> parameters.mod',rho  ));
system(sprintf('echo sigma = %0.14f; >> parameters.mod',sigma));

%% Steady state
%--------------------------------------------------------------------------

%Solving for steady state
zss = 1;
kss = ((1-beta+beta*delta)/(alpha*beta))^(1/(alpha-1));
css = kss^alpha-delta*kss;

%Creating steadystate_level.mod using system commands
delete steadystate_level.mod
system(sprintf('echo lnz = %0.14f; >> steadystate_level.mod',log(zss)));
system(sprintf('echo k   = %0.14f; >> steadystate_level.mod',kss     ));
system(sprintf('echo c   = %0.14f; >> steadystate_level.mod',css     ));

%Creating steadystate_log.mod using system commands
delete steadystate_log.mod
system(sprintf('echo lnz = %0.14f; >> steadystate_log.mod',log(zss)));
system(sprintf('echo lnk = %0.14f; >> steadystate_log.mod',log(kss)));
system(sprintf('echo lnc = %0.14f; >> steadystate_log.mod',log(css)));

%% Model in logs
%--------------------------------------------------------------------------

%Run Dynare
dynare model_log.mod noclearall
load dynarerocks,
decision_log = decision; 

%Cleaning
!del model_log.m
!del model_log_static.m
!del model_log_dynamic.m
!del model_log_results.mat
!del model_log.log
!rmdir model_log

%% Model in levels
%--------------------------------------------------------------------------

%Run Dynare
dynare model_level.mod noclearall
load dynarerocks
decision_lev = decision;

%Cleaning
!del model_level.m
!del model_level_static.m
!del model_level_dynamic.m
!del model_level_results.mat
!del model_level.log
!rmdir model_level

%Run Dynare
dynare model_level_1st.mod noclearall
load dynarerocks
decision_lev_1st = decision;

%Cleaning
!del model_level_1st.m
!del model_level_static_1st.m
!del model_level_dynamic_1st.m
!del model_level_results_1st.mat
!del model_level_1st.log
!rmdir model_level_1st

clc


%% Simulation
%--------------------------------------------------------------------------

%Settings
T = 10000;

%Stochastics
randn('state',20100930);                                            %#ok<RAND>
shocks = sigma*randn(T,1);

%Gaussian Hermite
gh.size = 5; [gh.e,gh.w] = hernodes(gh.size); temp = zeros(gh.size,1);

%Allocating memory and initialization
contruth = zeros(T,1); contruth(1)  = log(css); captruth = zeros(T,1); captruth(1) = log(kss);
con2ndno = zeros(T,1); con2ndno(1)  = css;      cap2ndno = zeros(T,1); cap2ndno(1) = kss;
con1stno = zeros(T,1); con1stno(1)  = css;      cap1stno = zeros(T,1); cap1stno(1) = kss;
                                                cap2ndp1 = zeros(T,1); cap2ndp1(1) = kss;
con2ndp2 = zeros(T,1); con2ndp2(1)  = css;      cap2ndp2 = zeros(T,1); cap2ndp2(1) = kss;
con2ndal = zeros(T,1); conp2ndal(1) = css;      cap2ndal = zeros(T,1); cap2ndal(1) = kss;
con1stal = zeros(T,1); con1stal(1)  = css;      cap1stal = zeros(T,1); cap1stal(1) = kss;
con1stAL = zeros(T,1); con1stAL(1)  = css;      cap1stAL = zeros(T,1); cap1stAL(1) = kss;
lnz      = zeros(T,1); lnz(1)      = 0;

%Simulation
for i = 2:T,
    disp(i)
    %Productivity
    lnz(i)      = rho*lnz(i-1)+shocks(i);
    
    %Truth (for gamma = delta = 1)
    khat        = captruth(i-1)-log(kss);
    captruth(i) = decision_log(:,2)'*[1,0,khat,lnz(i-1),shocks(i),0,0,0,0,0,0]';
    contruth(i) = decision_log(:,1)'*[1,0,khat,lnz(i-1),shocks(i),0,0,0,0,0,0]';
    
    %First-order
    khat        = cap1stno(i-1)-kss;
    cap1stno(i) = decision_lev(:,2)'*[1,-1,khat,lnz(i-1),shocks(i),0,0,0,0,0,0]';
    con1stno(i) = decision_lev(:,1)'*[1,-1,khat,lnz(i-1),shocks(i),0,0,0,0,0,0]';
    
    %Second-order
    khat        = cap2ndno(i-1)-kss;
    cap2ndno(i) = decision_lev(:,2)'*[1,0,khat,lnz(i-1),shocks(i),khat^2,khat*lnz(i-1),lnz(i-1)^2,shocks(i)^2,khat*shocks(i),lnz(i-1)*shocks(i)]';
    con2ndno(i) = decision_lev(:,1)'*[1,0,khat,lnz(i-1),shocks(i),khat^2,khat*lnz(i-1),lnz(i-1)^2,shocks(i)^2,khat*shocks(i),lnz(i-1)*shocks(i)]';
    
    %Second-order with pruning
    khat1       = cap2ndp1(i-1)-kss;
    cap2ndp1(i) = decision_lev(:,2)'*[1,0,khat1,lnz(i-1),shocks(i),0,0,0,0,0,0]';
    khat2       = cap2ndp2(i-1)-kss;
    cap2ndp2(i) = decision_lev(:,2)'*[1,0,khat2,lnz(i-1),shocks(i),khat1^2,khat1*lnz(i-1),lnz(i-1)^2,shocks(i)^2,khat1*shocks(i),lnz(i-1)*shocks(i)]';
    con2ndp2(i) = decision_lev(:,1)'*[1,0,khat2,lnz(i-1),shocks(i),khat1^2,khat1*lnz(i-1),lnz(i-1)^2,shocks(i)^2,khat1*shocks(i),lnz(i-1)*shocks(i)]';
        
    %First-order alternative - fixed point iteration - standard
%       for j = 1:gh.size,
%          khat     = cap1stal(i-1)-kss;
%          knew     = decision_lev_1st(:,2)'*[1,khat,lnz(i-1),shocks(i)]';
%          khatnew  = knew-kss;
%          shocknew = sqrt(2)*sigma*gh.e(j);
%          lnznew   = rho*lnz(i)+shocknew;
%          knewnew  = decision_lev_1st(:,2)'*[1,khatnew,lnz(i),shocknew,]';
%          cnew     = exp(lnznew)*knew^alpha+(1-delta)*knew-knewnew;
%          temp(j)  = (gh.w(j)/sqrt(pi))*beta*cnew^(-gamma)*(alpha*exp(lnznew)*knew^(alpha-1)+1-delta);
%       end
%      con1stal(i)=sum(temp)^(-1/gamma);
%      cap1stal(i) = exp(lnz(i))*cap1stal(i-1)^alpha+(1-delta)*cap1stal(i-1)-con1stal(i);
%  
    %First-order alternative - fixed point iteration - alternative
    
%      for j = 1:gh.size,
%          khat     = cap1stal(i-1)-kss;
%          knew     = decision_lev_1st(:,2)'*[1,khat,lnz(i-1),shocks(i)]';
%          cnow     = decision_lev_1st(:,1)'*[1,khat,lnz(i-1),shocks(i)]';
%          %cnow     = exp(lnz(i))*cap1stal(i-1)^alpha+(1-delta)*cap1stal(i-1)-knew;
%          khatnew  = knew-kss;
%          shocknew = sqrt(2)*sigma*gh.e(j);
%          lnznew   = rho*lnz(i)+shocknew;
%          knewnew  = decision_lev_1st(:,2)'*[1,khatnew,lnz(i),shocknew,]';
%          cnew     = exp(lnznew)*knew^alpha+(1-delta)*knew-knewnew;
%          temp(j)  = (gh.w(j)/sqrt(pi))*knew*beta*(cnew/cnow)^(-gamma)*(alpha*exp(lnznew)*knew^(alpha-1)+1-delta);
%      end
%      cap1stal(i) = (sum(temp));
%      con1stal(i) = exp(lnz(i))*cap1stal(i-1)^alpha+(1-delta)*cap1stal(i-1)-cap1stal(i);
     
     %[i cap1stal(i) exp(captruth(i))]
     %pause
     
% to do 1-step perturbation plus use 
% cap1stAL(i)  = fsolve(@(logknew) eulereq(logknew,logknow,lnznow),logknow,options);
% to do 2-step perturbation plus use 
% cap1stAL(i)  = fsolve(@(logknew) eulereq2(logknew,logknow,lnznow),logknow,options);
% to do 3-step perturbation plus use 
% cap1stAL(i)  = fsolve(@(logknew) eulereq1(logknew,logknow,lnznow),logknow,options);


    %First-order alternative - time iteration
    know        = cap1stAL(i-1);
    logknow     = log(know);
    lnznow      = lnz(i);
    cap1stAL(i)  = fsolve(@(logknew) eulereq(logknew,logknow,lnznow),logknow,options);
    cap1stAL(i)  = exp(cap1stAL(i));
    con1stAL(i)  = exp(lnznow)*know^alpha-cap1stAL(i);
%    if i > 2850
%        [cap1stAL(i) cap1stAL(i-1) con1stAL(i)]
%        pause
%    end
    %[cap1stAL(i) cap2ndno(i) exp(captruth(i))]
    %pause
end

    captruth = exp(captruth); %Transform truth from log to level.
    contruth = exp(contruth); %Transform truth from log to level.

%% accuracy   

    error_max_cap_ours_1  =  max( min([abs((cap1stAL-captruth)/mean(captruth)),abs((cap1stAL-captruth)./captruth)],[],2) );
    error_ave_cap_ours_1  = mean( min([abs((cap1stAL-captruth)/mean(captruth)),abs((cap1stAL-captruth)./captruth)],[],2) );
    error_max_con_ours_1  =  max( min([abs((con1stAL-contruth)/mean(contruth)),abs((con1stAL-contruth)./contruth)],[],2) );
    error_ave_con_ours_1  = mean( min([abs((con1stAL-contruth)/mean(contruth)),abs((con1stAL-contruth)./contruth)],[],2) );

    error_max_cap_pert_1  =  max( min([abs((cap1stno-captruth)/mean(captruth)),abs((cap1stno-captruth)./captruth)],[],2) );
    error_ave_cap_pert_1  = mean( min([abs((cap1stno-captruth)/mean(captruth)),abs((cap1stno-captruth)./captruth)],[],2) );
    error_max_con_pert_1  =  max( min([abs((con1stno-contruth)/mean(contruth)),abs((con1stno-contruth)./contruth)],[],2) );
    error_ave_con_pert_1  = mean( min([abs((con1stno-contruth)/mean(contruth)),abs((con1stno-contruth)./contruth)],[],2) );

    error_max_cap_pert_2  =  max( min([abs((cap2ndno-captruth)/mean(captruth)),abs((cap2ndno-captruth)./captruth)],[],2) );
    error_ave_cap_pert_2  = mean( min([abs((cap2ndno-captruth)/mean(captruth)),abs((cap2ndno-captruth)./captruth)],[],2) );
    error_max_con_pert_2  =  max( min([abs((con2ndno-contruth)/mean(contruth)),abs((con2ndno-contruth)./contruth)],[],2) );
    error_ave_con_pert_2  = mean( min([abs((con2ndno-contruth)/mean(contruth)),abs((con2ndno-contruth)./contruth)],[],2) );

    error_max_cap_pert_2p =  max( min([abs((cap2ndp2-captruth)/mean(captruth)),abs((cap2ndp2-captruth)./captruth)],[],2) );
    error_ave_cap_pert_2p = mean( min([abs((cap2ndp2-captruth)/mean(captruth)),abs((cap2ndp2-captruth)./captruth)],[],2) );
    error_max_con_pert_2p =  max( min([abs((con2ndp2-contruth)/mean(contruth)),abs((con2ndp2-contruth)./contruth)],[],2) );
    error_ave_con_pert_2p = mean( min([abs((con2ndp2-contruth)/mean(contruth)),abs((con2ndp2-contruth)./contruth)],[],2) );
   
disp([error_max_cap_ours_1  error_ave_cap_ours_1  error_max_con_ours_1  error_ave_con_ours_1 ])
disp([error_max_cap_pert_1  error_ave_cap_pert_1  error_max_con_pert_1  error_ave_con_pert_1 ])
disp([error_max_cap_pert_2  error_ave_cap_pert_2  error_max_con_pert_2  error_ave_con_pert_2 ])
disp([error_max_cap_pert_2p error_ave_cap_pert_2p error_max_con_pert_2p error_ave_con_pert_2p])

disp('ave stdev minimum maximum')
disp([mean(captruth) std(captruth) min(captruth) max(captruth)])
disp([mean(cap1stAL) std(cap1stAL) min(cap1stAL) max(cap1stAL)])
disp([mean(cap1stno) std(cap1stno) min(cap1stno) max(cap1stno)])
disp([mean(cap2ndno) std(cap2ndno) min(cap2ndno) max(cap2ndno)])
disp([mean(cap2ndp2) std(cap2ndp2) min(cap2ndp2) max(cap2ndp2)])

YY = [captruth cap1stAL cap1stno cap2ndno cap2ndp2 contruth con1stAL con1stno con2ndno con2ndp2];

%% Plotting simulation

for ii = 1:1
%    close all
%T1 = (ii-1)*100+1;
%T2 = (ii-1)*100+100;
T1 = 7750;
T2 = 7850;
%figure('Name','Simulation Capital')
subplot(2,1,1)
plot(T1:T2,[captruth(T1:T2),cap1stno(T1:T2),cap2ndno(T1:T2),cap1stAL(T1:T2),cap2ndp2(T1:T2)]')
title('Simulation')%, legend('Truth','1st-order','2nd-order','1st-order alternative TI','2nd-order pruned')
xlabel('\it{t}'), ylabel('\it{k_{t}}','Rotation',0)
axis([T1 T2 0 inf])
subplot(2,1,2)
plot(T1:T2,[(captruth(T1:T2)-captruth(T1:T2))./captruth(T1:T2),(cap1stno(T1:T2)-captruth(T1:T2))./captruth(T1:T2),(cap2ndno(T1:T2)-captruth(T1:T2))./captruth(T1:T2),(cap1stAL(T1:T2)-captruth(T1:T2))./captruth(T1:T2),(cap2ndp2(T1:T2)-captruth(T1:T2))./captruth(T1:T2)]')
title('Simulation error') %, legend('Truth','1st-order','2nd-order','1st-order alternative TI','2nd-order pruned')
xlabel('\it{t}'), ylabel('\it{error}','Rotation',90)
axis([T1 T2 -inf inf])


% figure('Name','Simulation Consumption')
% subplot(2,1,1)
% plot(T1:T2,[contruth(T1:T2),con1stno(T1:T2),con2ndno(T1:T2),con1stAL(T1:T2),con2ndp2(T1:T2)]')
% title('Simulation')%, legend('Truth','1st-order','2nd-order','1st-order alternative TI','2nd-order pruned')
% xlabel('\it{t}'), ylabel('\it{k_{t}}','Rotation',0)
% axis([T1 T2 0 inf])
% subplot(2,1,2)
% plot(T1:T2,[(contruth(T1:T2)-contruth(T1:T2))./contruth(T1:T2),(con1stno(T1:T2)-contruth(T1:T2))./contruth(T1:T2),(con2ndno(T1:T2)-contruth(T1:T2))./contruth(T1:T2),(con1stAL(T1:T2)-contruth(T1:T2))./contruth(T1:T2),(con2ndp2(T1:T2)-contruth(T1:T2))./contruth(T1:T2)]')
% title('Simulation error')%, legend('Truth','1st-order','2nd-order','1st-order alternative TI','2nd-order pruned')
% xlabel('\it{t}'), ylabel('\it{error}','Rotation',90)
% axis([T1 T2 -inf inf])

%
% nicer graph
%

close
plot(T1:T2,captruth(T1:T2),'MarkerSize',10,'Marker','.','Color',[0 0 0]);
hold all
plot(T1:T2,cap1stAL(T1:T2),'LineWidth',2.5,'LineStyle',':','Color',[1 0 0]);
hold all
plot(T1:T2,cap1stno(T1:T2),'Color',[0 0 1]);
hold all
plot(T1:T2,cap2ndno(T1:T2),'LineWidth',2,'LineStyle','--','Color',[0 1 0]);
hold all
plot(T1:T2,cap2ndp2(T1:T2),'Marker','.','Color',[1 0 1]);
hold all
plot(T1:T2,zeros(T2-T1+1),':k','LineWidth',0.5)
xlabel('time')
ylabel('$k_{t}$','Interpreter','latex','Rotation',90)
axis([T1 T2 -0.5 7])
box off


end

%% Which one is in predicting changes?
%--------------------------------------------------------------------------

% %Metric 1: Sign of first-difference right?
% captruthdif = diff(captruth); cap2ndnodif = diff(cap2ndno);
% cap2ndp2dif = diff(cap2ndp2); cap1stALdif = diff(cap1stAL);
% sum_sign_right_normal = sum(sign(cap2ndnodif)==sign(captruthdif))/length(captruthdif)*100;
% sum_sign_right_pruned = sum(sign(cap2ndp2dif)==sign(captruthdif))/length(captruthdif)*100;
% sum_sign_right_altern = sum(sign(cap1stALdif)==sign(captruthdif))/length(captruthdif)*100;
% 
% fprintf('\n')
% disp('Sign of first-difference right?')
% fprintf('\n')
% disp('Normal simulation:         ')
% disp(sum_sign_right_normal)
% fprintf('\n')
% disp('Pruned simulation:         ')
% disp(sum_sign_right_pruned)
% fprintf('\n')
% disp('Alternative simulation:    ')
% disp(sum_sign_right_altern)




