%% 
%
% Nonlinear and stable perturbation-based approximations
% Wouter J. Den Haan and Joris De Wind
%
% programs for modified Deaton model
%
% calculates statistics for Table 1
%
%%%
%--------------------------------------------------------------------------

%Cleaning
clear all; close all; clc

%% Parameters
%--------------------------------------------------------------------------

%Structural
par.beta  = 0.9;
par.r     = 0.03;
par.gamma = 3;
par.mu    = 0.4;
par.sigma = 0.1;

%Penalty
par.eta0 = 20;
par.eta1 =  0.04597796327161;
par.eta2 = -0.00362930657591;

%% Grid
%--------------------------------------------------------------------------

%Grid for cash on hand
xgrid = (1:0.001:2.5);

%Gaussian Hermite nodes
gh.size     = 30;
[gh.e,gh.w] = hernodes(gh.size);

%Options
options = optimset('Display','Off','Maxiter',1E3,'TolX',1E-10,'TolFun',1E-10,'UseParallel','Always');

%UseParallel is not supported by fsolve and can thus be deleted!

%% Income
%--------------------------------------------------------------------------

%State random number generator
randn('state',11051981)                                          %#ok<RAND>

%Number of periods
Time = 10000;

%Time = 5000 for simulations while T = 10000 for accuracy!

%Income
y = exp(par.mu+par.sigma*randn(1,Time));

%% First-order
%--------------------------------------------------------------------------

%Loading policy function
load d1_calibration1
Apol1 = dyn_ss(2,1)+dyn_g_1(2,1)*(xgrid-dyn_ss(2,1)-dyn_ss(3,1));

%Simulation 1 - normal
Asim1 = zeros(1,Time+1);
for i = 1:Time,
    Asim1(1,i+1) = interp1(xgrid,Apol1,Asim1(1,i)+y(i),'*linear','extrap');
end
csim1 = Asim1(1,1:end-1)+y-Asim1(1,2:end)/(1+par.r);
    
%Simulation 2 - dynamic Euler equation test, FIXED POINT
Asim1_test = zeros(1,Time+1);
for i = 1:Time,
    Atemp             = interp1(xgrid,Apol1,Asim1_test(1,i)+y(i),'*linear','extrap');
    Expec             = (1+par.r)*par.beta*sum(gh.w.*(exp(par.mu+sqrt(2)*par.sigma*gh.e)+Atemp-...
                        interp1(xgrid,Apol1,exp(par.mu+sqrt(2)*par.sigma*gh.e)+Atemp,'*linear',...
                        'extrap')/(1+par.r)).^(-par.gamma))/sqrt(pi);
    Asim1_test(1,i+1) = fsolve(@(x) -par.gamma*log(Asim1_test(1,i)+y(i)-x/(1+par.r))-...
                        log(Expec+par.eta2+par.eta1*exp(-par.eta0*x)),Atemp,options);
end
csim1_test = Asim1_test(1,1:end-1)+y-Asim1_test(1,2:end)/(1+par.r);

%Simulation 3 - dynamic Euler equation test, TIME ITERATION
Asim1_time1 = zeros(1,Time+1);
for i = 1:Time,
    initial = interp1(xgrid,Apol1,Asim1_time1(1,i)+y(i),'*linear','extrap'); %Initial guess is first-order perturbation solution!
    Asim1_time1(1,i+1) = fsolve(@(achoice) alterr(achoice,Asim1_time1(1,i)+y(i),xgrid,Apol1,par,gh),initial,options);
end
csim1_time1 = Asim1_time1(1,1:end-1)+y-Asim1_time1(1,2:end)/(1+par.r);

%Simulation 4 - dynamic Euler equation test, TWO PERIOD TIME ITERATION
step = 2;
Asim1_time2 = zeros(1,Time+1);
for i = 1:Time,
    initial = interp1(xgrid,Apol1,Asim1_time2(1,i)+y(i),'*linear','extrap'); %Initial guess is first-order perturbation solution!
    Asim1_time2(1,i+1) = fsolve(@(achoice) alterrmul(achoice,Asim1_time2(1,i)+y(i),xgrid,Apol1,par,gh,step,options),initial,options);
    disp(i)
end
csim1_time2 = Asim1_time2(1,1:end-1)+y-Asim1_time2(1,2:end)/(1+par.r);

%% First-order with correction term
%--------------------------------------------------------------------------

%Loading policy function
load d1_calibration1
load correction
Apol12 = dyn_ss(2,1)+decision(2,1)+dyn_g_1(2,1)*(xgrid-dyn_ss(2,1)-dyn_ss(3,1)); %Includes correction term!

%Simulation 1 - normal
Asim12 = zeros(1,Time+1);
for i = 1:Time,
    Asim12(1,i+1) = interp1(xgrid,Apol12,Asim12(1,i)+y(i),'*linear','extrap');
end
csim12 = Asim12(1,1:end-1)+y-Asim12(1,2:end)/(1+par.r);
    
%Simulation 2 - dynamic Euler equation test, FIXED POINT
Asim12_test = zeros(1,Time+1);
for i = 1:Time,
    Atemp              = interp1(xgrid,Apol12,Asim12_test(1,i)+y(i),'*linear','extrap');
    Expec              = (1+par.r)*par.beta*sum(gh.w.*(exp(par.mu+sqrt(2)*par.sigma*gh.e)+Atemp-...
                         interp1(xgrid,Apol12,exp(par.mu+sqrt(2)*par.sigma*gh.e)+Atemp,'*linear',...
                         'extrap')/(1+par.r)).^(-par.gamma))/sqrt(pi);
    Asim12_test(1,i+1) = fsolve(@(x) -par.gamma*log(Asim12_test(1,i)+y(i)-x/(1+par.r))-...
                         log(Expec+par.eta2+par.eta1*exp(-par.eta0*x)),Atemp,options);
end
csim12_test = Asim12_test(1,1:end-1)+y-Asim12_test(1,2:end)/(1+par.r);

%Simulation 3 - dynamic Euler equation test, TIME ITERATION
Asim12_time1 = zeros(1,Time+1);
for i = 1:Time,
    initial = interp1(xgrid,Apol12,Asim12_time1(1,i)+y(i),'*linear','extrap'); %Initial guess is first-order perturbation solution!
    Asim12_time1(1,i+1) = fsolve(@(achoice) alterr(achoice,Asim12_time1(1,i)+y(i),xgrid,Apol12,par,gh),initial,options);
end
csim12_time1 = Asim12_time1(1,1:end-1)+y-Asim12_time1(1,2:end)/(1+par.r);

%Simulation 4 - dynamic Euler equation test, TWO PERIOD TIME ITERATION
step = 2;
Asim12_time2 = zeros(1,Time+1);
for i = 1:Time,
    initial = interp1(xgrid,Apol12,Asim12_time2(1,i)+y(i),'*linear','extrap'); %Initial guess is first-order perturbation solution!
    Asim12_time2(1,i+1) = fsolve(@(achoice) alterrmul(achoice,Asim12_time2(1,i)+y(i),xgrid,Apol12,par,gh,step,options),initial,options);
    disp(i)
end
csim12_time2 = Asim12_time2(1,1:end-1)+y-Asim12_time2(1,2:end)/(1+par.r);

%% Second-order without pruning
%--------------------------------------------------------------------------

%Loading policy function
load d2_calibration1
Apol2 = dyn_ss(2,1)+dyn_g_1(2,1)*(xgrid-dyn_ss(2,1)-dyn_ss(3,1))+...
                    dyn_g_2(2,1)*(xgrid-dyn_ss(2,1)-dyn_ss(3,1)).^2;

%Simulation 1 - normal
Asim2 = zeros(1,Time+1);
for i = 1:Time,
    Asim2(1,i+1) = interp1(xgrid,Apol2,Asim2(1,i)+y(i),'*linear','extrap');
end
csim2 = Asim2(1,1:end-1)+y-Asim2(1,2:end)/(1+par.r);

% %Simulation 2 - dynamic Euler equation test, FIXED POINT
% Asim2_test = zeros(1,Time+1);
% for i = 1:Time,
%     Atemp             = interp1(xgrid,Apol2,Asim2_test(1,i)+y(i),'*linear','extrap');
%     Expec             = (1+par.r)*par.beta*sum(gh.w.*(exp(par.mu+sqrt(2)*par.sigma*gh.e)+Atemp-...
%                         interp1(xgrid,Apol2,exp(par.mu+sqrt(2)*par.sigma*gh.e)+Atemp,'*linear',...
%                         'extrap')/(1+par.r)).^(-par.gamma))/sqrt(pi);
%     Asim2_test(1,i+1) = fsolve(@(x) -par.gamma*log(Asim2_test(1,i)+y(i)-x/(1+par.r))-...
%                         log(Expec+par.eta2+par.eta1*exp(-par.eta0*x)),Atemp,options);
% end
% csim2_test = Asim2_test(1,1:end-1)+y-Asim2_test(1,2:end)/(1+par.r);

%% Second-order with pruning
%--------------------------------------------------------------------------

%Loading policy function
load d2_calibration1
Apollin2 = dyn_ss(2,1)+dyn_g_1(2,1)*(xgrid-dyn_ss(2,1)-dyn_ss(3,1));
Apolnon2 =             dyn_g_2(2,1)*(xgrid-dyn_ss(2,1)-dyn_ss(3,1)).^2;

%Simulation 1 - normal
Asimlin2 = zeros(1,Time+1);
Asimpru2 = zeros(1,Time+1);
for i = 1:Time,
    Asimlin2(1,i+1) = interp1(xgrid,Apollin2,Asimlin2(1,i)+y(i),'*linear','extrap');
    Asimpru2(1,i+1) = interp1(xgrid,Apollin2,Asimpru2(1,i)+y(i),'*linear','extrap')+...
                      interp1(xgrid,Apolnon2,Asimlin2(1,i)+y(i),'*linear','extrap');
end
csimpru2 = Asimpru2(1,1:end-1)+y-Asimpru2(1,2:end)/(1+par.r);

% %Simulation 2 - dynamic Euler equation test, FIXED POINT
% Asimlin2_test = zeros(1,Time+1);
% Asimpru2_test = zeros(1,Time+1);
% for i = 1:Time,
%     Atemplin             = interp1(xgrid,Apollin2,Asimlin2_test(1,i)+y(i),'*linear','extrap');
%     Atemppru             = interp1(xgrid,Apollin2,Asimpru2_test(1,i)+y(i),'*linear','extrap')+...
%                            interp1(xgrid,Apolnon2,Asimlin2_test(1,i)+y(i),'*linear','extrap');
%     Expeclin             = (1+par.r)*par.beta*sum(gh.w.*(exp(par.mu+sqrt(2)*par.sigma*gh.e)+Atemplin-...
%                            interp1(xgrid,Apollin2,exp(par.mu+sqrt(2)*par.sigma*gh.e)+Atemplin,'*linear',...
%                            'extrap')/(1+par.r)).^(-par.gamma))/sqrt(pi);
%     Expecpru             = (1+par.r)*par.beta*sum(gh.w.*(exp(par.mu+sqrt(2)*par.sigma*gh.e)+Atemppru-...
%                            (interp1(xgrid,Apollin2,exp(par.mu+sqrt(2)*par.sigma*gh.e)+Atemppru,'*linear','extrap')+...
%                            interp1(xgrid,Apolnon2,exp(par.mu+sqrt(2)*par.sigma*gh.e)+Atemplin,'*linear','extrap'))...
%                            /(1+par.r)).^(-par.gamma))/sqrt(pi);
%     Asimlin2_test(1,i+1) = fsolve(@(x) -par.gamma*log(Asimlin2_test(1,i)+y(i)-x/(1+par.r))-...
%                            log(Expeclin+par.eta2+par.eta1*exp(-par.eta0*x)),Atemplin,options);
%     Asimpru2_test(1,i+1) = fsolve(@(x) -par.gamma*log(Asimpru2_test(1,i)+y(i)-x/(1+par.r))-...
%                            log(Expecpru+par.eta2+par.eta1*exp(-par.eta0*x)),Atemppru,options);
% end
% csimpru2_test = Asimpru2_test(1,1:end-1)+y-Asimpru2_test(1,2:end)/(1+par.r);

%% Truth
%--------------------------------------------------------------------------

%Loading policy function
load dtruth
Apoltruth = interp1(x_policy,Ap_policy,xgrid,'*linear','extrap');

%Simulation 1 - normal
Asimtruth = zeros(1,Time+1);
for i = 1:Time,
    Asimtruth(1,i+1) = interp1(xgrid,Apoltruth,Asimtruth(1,i)+y(i),'*linear','extrap');
end
csimtruth = Asimtruth(1,1:end-1)+y-Asimtruth(1,2:end)/(1+par.r);
    
%Simulation 2 - dynamic Euler equation test, FIXED POINT
Asimtruth_test = zeros(1,Time+1);
for i = 1:Time,
    Atemp                 = interp1(xgrid,Apoltruth,Asimtruth_test(1,i)+y(i),'*linear','extrap');
    Expec                 = (1+par.r)*par.beta*sum(gh.w.*(exp(par.mu+sqrt(2)*par.sigma*gh.e)+Atemp-...
                            interp1(xgrid,Apoltruth,exp(par.mu+sqrt(2)*par.sigma*gh.e)+Atemp,'*linear',...
                            'extrap')/(1+par.r)).^(-par.gamma))/sqrt(pi);
    Asimtruth_test(1,i+1) = fsolve(@(x) -par.gamma*log(Asimtruth_test(1,i)+y(i)-x/(1+par.r))-...
                            log(Expec+par.eta2+par.eta1*exp(-par.eta0*x)),Atemp,options);
end
csimtruth_test = Asimtruth_test(1,1:end-1)+y-Asimtruth_test(1,2:end)/(1+par.r);

%% Plotting
%--------------------------------------------------------------------------

% figpolfun = figure('Name','Simulations','Position',[8 40 1280/2.05 1024*0.92*0.55]);
% axes1 = axes('Parent',figpolfun,'Position',[0.08 0.10/0.55 0.88 0.40/0.55]); hold all
% plot(3610:3660,Asimtruth  (3610:3660),'-'  ,'LineWidth',0.5,'DisplayName','truth'                       )
% plot(3610:3660,Asim1      (3610:3660),'--' ,'LineWidth',0.5,'DisplayName','1^{st}-order'                )
% plot(3610:3660,Asim1_time1(3610:3660),':'  ,'LineWidth',1  ,'DisplayName','1^{st}-order, one period time iteration')
% plot(3610:3660,Asim1_time2(3610:3660),':'  ,'LineWidth',2  ,'DisplayName','1^{st}-order, two period time iteration')
% plot(3610:3660,Asim1_test (3610:3660),'--' ,'LineWidth',1  ,'DisplayName','1^{st}-order, fixed point'   )
% plot(3610:3660,Asimpru2   (3610:3660),'--' ,'LineWidth',0.5,'DisplayName','2^{nd}-order, pruned'        )
% xlabel('\it{t}'), ylabel('\it{a_{t}}','Rotation',0), legend('Location','NorthWest')
% legend('BoxOff'), axis([3610 3660 -0.1 0.7])
% set(axes1,'YTick',[-0.1 0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7],...
%      'YTickLabel',[-0.1 0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7]);

%% Accuracy
%--------------------------------------------------------------------------

disp('1 regular')
max (min([abs((Asim1-Asimtruth)/mean(Asimtruth));abs((Asim1-Asimtruth)./Asimtruth)],[],1))
mean(min([abs((Asim1-Asimtruth)/mean(Asimtruth));abs((Asim1-Asimtruth)./Asimtruth)],[],1))

disp('1 regular with correction term')
max (min([abs((Asim12-Asimtruth)/mean(Asimtruth));abs((Asim12-Asimtruth)./Asimtruth)],[],1))
mean(min([abs((Asim12-Asimtruth)/mean(Asimtruth));abs((Asim12-Asimtruth)./Asimtruth)],[],1))

disp('1 time iteration, one period ahead')
max (min([abs((Asim1_time1-Asimtruth)/mean(Asimtruth));abs((Asim1_time1-Asimtruth)./Asimtruth)],[],1))
mean(min([abs((Asim1_time1-Asimtruth)/mean(Asimtruth));abs((Asim1_time1-Asimtruth)./Asimtruth)],[],1))

disp('1 time iteration, one period ahead with correction')
max (min([abs((Asim12_time1-Asimtruth)/mean(Asimtruth));abs((Asim12_time1-Asimtruth)./Asimtruth)],[],1))
mean(min([abs((Asim12_time1-Asimtruth)/mean(Asimtruth));abs((Asim12_time1-Asimtruth)./Asimtruth)],[],1))

disp('1 time iteration, two periods ahead')
max (min([abs((Asim1_time2-Asimtruth)/mean(Asimtruth));abs((Asim1_time2-Asimtruth)./Asimtruth)],[],1))
mean(min([abs((Asim1_time2-Asimtruth)/mean(Asimtruth));abs((Asim1_time2-Asimtruth)./Asimtruth)],[],1))

disp('1 time iteration, two periods ahead with correction')
max (min([abs((Asim12_time2-Asimtruth)/mean(Asimtruth));abs((Asim12_time2-Asimtruth)./Asimtruth)],[],1))
mean(min([abs((Asim12_time2-Asimtruth)/mean(Asimtruth));abs((Asim12_time2-Asimtruth)./Asimtruth)],[],1))

disp('1 fixed point iteration')
max (min([abs((Asim1_test-Asimtruth)/mean(Asimtruth));abs((Asim1_test-Asimtruth)./Asimtruth)],[],1))
mean(min([abs((Asim1_test-Asimtruth)/mean(Asimtruth));abs((Asim1_test-Asimtruth)./Asimtruth)],[],1))

disp('1 fixed point iteration with correction')
max (min([abs((Asim12_test-Asimtruth)/mean(Asimtruth));abs((Asim12_test-Asimtruth)./Asimtruth)],[],1))
mean(min([abs((Asim12_test-Asimtruth)/mean(Asimtruth));abs((Asim12_test-Asimtruth)./Asimtruth)],[],1))

disp('2 regular')
max (min([abs((Asim2-Asimtruth)/mean(Asimtruth));abs((Asim2-Asimtruth)./Asimtruth)],[],1))
mean(min([abs((Asim2-Asimtruth)/mean(Asimtruth));abs((Asim2-Asimtruth)./Asimtruth)],[],1))

disp('2 pruning')
max (min([abs((Asimpru2-Asimtruth)/mean(Asimtruth));abs((Asimpru2-Asimtruth)./Asimtruth)],[],1))
mean(min([abs((Asimpru2-Asimtruth)/mean(Asimtruth));abs((Asimpru2-Asimtruth)./Asimtruth)],[],1))

disp('truth')
max (min([abs((Asimtruth_test-Asimtruth)/mean(Asimtruth));abs((Asimtruth_test-Asimtruth)./Asimtruth)],[],1))
mean(min([abs((Asimtruth_test-Asimtruth)/mean(Asimtruth));abs((Asimtruth_test-Asimtruth)./Asimtruth)],[],1))

save results_for_tables

% %Descriptive statistics
% disp([mean(Asimtruth)    std(Asimtruth)    min(Asimtruth)    max(Asimtruth)    ])
% disp([mean(Asim1)        std(Asim1)        min(Asim1)        max(Asim1)        ])
% disp([mean(Asim12)       std(Asim12)       min(Asim12)       max(Asim12)       ])
% disp([mean(Asim1_time1)  std(Asim1_time1)  min(Asim1_time1)  max(Asim1_time1)  ])
% disp([mean(Asim12_time1) std(Asim12_time1) min(Asim12_time1) max(Asim12_time1) ])
% disp([mean(Asim1_time2)  std(Asim1_time2)  min(Asim1_time2)  max(Asim1_time2)  ])
% disp([mean(Asim12_time2) std(Asim12_time2) min(Asim12_time2) max(Asim12_time2) ])
% disp([mean(Asim1_test)   std(Asim1_test)   min(Asim1_test)   max(Asim1_test)   ])
% disp([mean(Asim12_test)  std(Asim12_test)  min(Asim12_test)  max(Asim12_test)  ])
% zdisp([mean(Asim2)        std(Asim2)        min(Asim2)        max(Asim2)        ])
% disp([mean(Asimpru2)     std(Asimpru2)     min(Asimpru2)     max(Asimpru2)     ])