%% 
%
% Nonlinear and stable perturbation-based approximations
% Wouter J. Den Haan and Joris De Wind
%
% Matching Model programs
%
%   Calculates statistics for Table 1
%   in the directory with your Dynare files you have to replace the disp_dr
%   file with the disp_dr file found in this directory

%%%
%
%   The constraint
%
%               matches =< min{unemployed,vacancies}
%
%   is eliminated, which means that firms and workers can end up with
%   more than one job. If one wants to impose the constraint, one needs
%   another matching function (like in DRW) to avoid the discontinuity.
%
%   JEDC revision
%
% Joris de Wind (October, 2010)
%--------------------------------------------------------------------------

%Cleaning
clear all; close all; clc
global options

%% Parameters and steady state
%--------------------------------------------------------------------------
iplotpert2 =1; % if equal to 0 then not regular perturbation
%Fix some parameters
par.alpha = 1;
par.beta  = 0.99;
par.gamma = 4.5;
par.phi   = 0.5;
par.rho_z = 0.95;
par.sigma = 0.007;
%par.omega = 0.973; % max is 0.9776 to keep current-period surplus >0
par.omega = 0.96; % max is 0.9776 to keep current-period surplus >0
%Fix some steady states
steady.z  = 0;
steady.n  = 0.95;
steady.lw = 0.70;
steady.lf = 0.70;

%Calculate some parameters and steady states
steady.u  = 1-steady.n;
steady.v  = steady.u*steady.lw/steady.lf;
par.rho_x = steady.u*steady.lw/steady.n;
par.phi0  = steady.lf*(steady.v/steady.u)^par.phi;
par.wage  = par.omega*par.alpha*steady.n^(par.alpha-1);
steady.p  = par.alpha*steady.n^(par.alpha-1)-par.wage;
steady.g  = par.beta*steady.p/(1-par.beta*(1-par.rho_x));
par.cost  = par.phi0*(steady.u/steady.v)^par.phi*steady.g;
steady.c  = steady.n^par.alpha-par.cost*steady.v;

%% Grid
%--------------------------------------------------------------------------

%Labor
n.size = 1E4;
n.low  = 0.6;
n.high = 0.99;
n.step = (n.high-n.low)/(n.size-1);
n.grid = (n.low:n.step:n.high)';

%Productivity
z.size = 2;
z.low  = -sqrt(par.sigma^2/(1-par.rho_z^2));
z.high =  sqrt(par.sigma^2/(1-par.rho_z^2));
z.step = (z.high-z.low)/(z.size-1);
z.grid = (z.low:z.step:z.high)';

%Transition probabilities
q = (par.rho_z+1)/2;
p = [q,1-q;1-q,q];

%Mesh
[grid.n,grid.z] = meshgrid(n.grid,z.grid);

%% Initials
%--------------------------------------------------------------------------

%Creating parameters.mod using system commands
delete parameters_matching.mod
system(sprintf('echo alpha = %0.14f; >> parameters_matching.mod',par.alpha ));
system(sprintf('echo beta  = %0.14f; >> parameters_matching.mod',par.beta  ));
system(sprintf('echo gamma = %0.14f; >> parameters_matching.mod',par.gamma ));
system(sprintf('echo phi   = %0.14f; >> parameters_matching.mod',par.phi   ));
system(sprintf('echo rho_z = %0.14f; >> parameters_matching.mod',par.rho_z ));
system(sprintf('echo sigma = %0.14f; >> parameters_matching.mod',par.sigma ));
system(sprintf('echo rho_x = %0.14f; >> parameters_matching.mod',par.rho_x ));
system(sprintf('echo phi0  = %0.14f; >> parameters_matching.mod',par.phi0  ));
system(sprintf('echo wage  = %0.14f; >> parameters_matching.mod',par.wage  ));
system(sprintf('echo cost  = %0.14f; >> parameters_matching.mod',par.cost  ));

%Creating steadystate.mod using system commands
delete steadystate_matching.mod
system(sprintf('echo z = %0.14f; >> steadystate_matching.mod',steady.z ));
system(sprintf('echo n = %0.14f; >> steadystate_matching.mod',steady.n ));
system(sprintf('echo u = %0.14f; >> steadystate_matching.mod',steady.u ));
system(sprintf('echo v = %0.14f; >> steadystate_matching.mod',steady.v ));
system(sprintf('echo p = %0.14f; >> steadystate_matching.mod',steady.p ));
system(sprintf('echo g = %0.14f; >> steadystate_matching.mod',steady.g ));
system(sprintf('echo c = %0.14f; >> steadystate_matching.mod',steady.c ));

%Run Dynare
dynare matching.mod noclearall
load dynarerocks

%Cleaning
!del matching.m
!del matching_static.m
!del matching_dynamic.m
!del matching_results.mat
!del matching.log
!rmdir matching

load dynarerocks

%Policy function
pert.v = zeros(z.size,n.size);
pert.n = zeros(z.size,n.size);
pert.g = zeros(z.size,n.size);
for i = 1:z.size,
    for j = 1:n.size,
        nhat = grid.n(i,j)-steady.n; zhat = grid.z(i,j);
        pert.v(i,j) = decision(:,1)'*[1;0;nhat;0;zhat;nhat^2;0;0;zhat^2;nhat*zhat;0]; %2nd-order
        pert.n(i,j) = decision(:,2)'*[1;0;nhat;0;zhat;nhat^2;0;0;zhat^2;nhat*zhat;0];
        pert.g(i,j) = decision(:,3)'*[1;0;nhat;0;zhat;nhat^2;0;0;zhat^2;nhat*zhat;0];
    end
end

%Options
approxchoice  = 'continuationvalue';
initials      = 'previous';     %For large grids "fresh" doesn't work, because
                                %the perturbation solution is too weird.

%Switching through options
switch approxchoice
    case 'continuationvalue'
        switch initials
            case 'fresh'
                coef = pert.g;
            case 'previous'
                load pol_g
                coef = vpol(grid.z,grid.n);
        end
    case 'vacancies'
        switch initials
            case 'fresh'
                coef = pert.v;
            case 'previous'
                load pol_v
                coef = vpol(grid.z,grid.n);
        end
        disp('This options is not yet implemented!')
        return
end

%% Projection for continuation value
%--------------------------------------------------------------------------

%Settings
maxiter   = 1E5;
criterium = 1E-12;
update    = 0.5;
iteration = 0;
dif       = 1;

%Projection
tic
while (iteration < maxiter && dif > criterium)
    
    %Now
    unow = 1-grid.n;
    gnow = coef;
    vnow = unow.*(par.phi0*gnow/par.cost).^(1/par.phi);
    nnow = (1-par.rho_x)*grid.n+par.phi0*unow.^par.phi.*vnow.^(1-par.phi);
    cnow = exp(grid.z).*grid.n.^par.alpha-par.cost*vnow;
    
    %Next (partly vectorized)
    rhs = zeros(z.size,n.size,z.size); znext = zeros(size(grid.n));
    for j = 1:z.size,
        znext(:,:) = grid.z(j);
        unext      = 1-nnow;
        gnext      = interpol2(z.grid,n.grid,coef,znext,nnow);
        vnext      = unext.*(par.phi0*gnext/par.cost).^(1/par.phi);
        cnext      = exp(znext).*(nnow.^par.alpha)-par.cost*vnext;
        pnext      = par.alpha*exp(znext).*nnow.^(par.alpha-1)-par.wage;
        rhs(:,:,j) = par.beta*repmat(p(:,j),1,n.size).*(cnext./cnow).^...
                     -par.gamma.*(pnext+(1-par.rho_x)*gnext);
    end
    
    %Continuation value
    gnow = sum(rhs,3);
    
    %Update
    dif       = max(max(abs(gnow-coef)));
    coef      = update*gnow+(1-update)*coef;
    iteration = iteration+1;
    
    %Print
    fprintf('\n'), disp('Dif = ...'), disp(dif), fprintf('\n')
    
end
toc

%Policy function
vpol = @(znow,nlag) interpol2(z.grid,n.grid,coef,znow,nlag);
save pol_g vpol



%% Plotting
%--------------------------------------------------------------------------

%Projection
switch approxchoice
    case 'continuationvalue'
        proj.v = (1-grid.n).*(par.phi0*coef/par.cost).^(1/par.phi); 
        proj.n = (1-par.rho_x)*grid.n+par.phi0*(1-grid.n).^par.phi.*proj.v.^(1-par.phi);
    case 'vacancies'
        proj.v = coef;
        proj.n = (1-par.rho_x)*grid.n+par.phi0*(1-grid.n).^par.phi.*proj.v.^(1-par.phi);        
end

close all
%Figure projection
figure('Name','Policy function projection')
hold all
plot(n.grid',proj.n(1,:),'b','LineWidth',2,'LineStyle',':')
plot(n.grid',pert.n(1,:),'b','LineWidth',2)
plot(n.grid',proj.n(2,:),'b','LineWidth',2,'LineStyle',':')
plot(n.grid',pert.n(2,:),'b','LineWidth',2)
plot(n.grid',n.grid','k','LineWidth',1)
%title('Policy function projection')
legend('"truth"','second-order perturbation')
xlabel('\it{n_{t-1}}'), ylabel('\it{n_{t}}','Rotation',90)
axis([0.85 0.99 0.85 0.99])
box off

%Saving
print('-depsc2','-painters','Policy function projection')
fixPSlinestyle('Policy function projection.eps')
saveas(gcf,'Policy function projection','fig')

%Figure perturbation
% figure('Name','Policy function perturbation')
% hold all
% plot(n.grid',pert.n(1,:),'r','LineWidth',2)
% plot(n.grid',pert.n(2,:),'b','LineWidth',2)
% plot(n.grid',n.grid','k','LineWidth',2)
% title('Policy function perturbation')
% legend('Perturbation low','Perturbation high','45-degree','Location','NorthWest')
% xlabel('\it{n_{t-1}}'), ylabel('\it{n_{t}}','Rotation',0)
% axis([0.6 1 0.5 1])
% box off

%Saving
print('-depsc2','-painters','Policy function perturbation')
fixPSlinestyle('Policy function perturbation.eps')
saveas(gcf,'Policy function perturbation','fig')


%% Accuracy
%--------------------------------------------------------------------------

% %New grid
% n.size = 2*n.size; n.step = (n.high-n.low)/(n.size-1); n.grid = (n.low:n.step:n.high)';
% [grid.n,grid.z] = meshgrid(n.grid,z.grid);
% 
% %Now
% unow = 1-grid.n;
% gnow = vpol(grid.z,grid.n);
% vnow = unow.*(par.phi0*gnow/par.cost).^(1/par.phi);
% nnow = (1-par.rho_x)*grid.n+par.phi0*unow.^par.phi.*vnow.^(1-par.phi);
% cnow = exp(grid.z).*grid.n.^par.alpha-par.cost*vnow;
% 
% %LHS Euler equation
% gnow_lhs = gnow;
% 
% %Next (partly vectorized)
% rhs = zeros(z.size,n.size,n.size); znext = zeros(size(grid.n));
% for j = 1:z.size,
%     znext(:,:) = grid.z(j);
%     unext      = 1-nnow;
%     gnext      = vpol(znext,nnow);
%     vnext      = unext.*(par.phi0*gnext/par.cost).^(1/par.phi);
%     cnext      = exp(znext).*(nnow.^par.alpha)-par.cost*vnext;
%     pnext      = par.alpha*exp(znext).*nnow.^(par.alpha-1)-par.wage;
%     rhs(:,:,j) = par.beta*repmat(p(:,j),1,n.size).*(cnext./cnow).^...
%                  -par.gamma.*(pnext+(1-par.rho_x)*gnext);
% end
% 
% %RHS Euler equation
% gnow_rhs = sum(rhs,3);
% 
% %Euler equation error
% errg = gnow_rhs-gnow_lhs;
% 
% %Print
% fprintf('\n'), disp('Max abs error g = ...'), disp(max(max(abs(errg)))), fprintf('\n')
% 
% %Figure
% figure('Name','Accuracy')
% hold all
% plot(n.grid',errg(1,:))
% plot(n.grid',errg(2,:))
% title('Accuracy'), legend('Low','High')
% xlabel('\it{n_{t-1}}'), ylabel('\it{error g_{t}}')
% box off
% 
% %Saving
% print('-depsc2','-painters','Accuracy')
% fixPSlinestyle('Accuracy.eps')
% saveas(gcf,'Accuracy','fig')

%% Simulation
%--------------------------------------------------------------------------

%Settings
T = 10000;
rand('seed',20101010)

%Productivity
index = zeros(T,1); index(1) = 1;
for i = 2:T,
    index(i) = find(mnrnd(1,p(index(i-1),:)));
end
lnz = z.grid(index);

%Options for first-order alternative
options = optimset('Display','off','MaxIter',1E3,'TolFun',1E-10,'TolX',1E-10);

%Allocating memory and initialization
sim.proj         = zeros(T,1); sim.proj(1)         = steady.n;
sim.pert.norm.n1 = zeros(T,1); sim.pert.norm.n1(1) = steady.n;
sim.pert.norm.n2 = zeros(T,1); sim.pert.norm.n2(1) = steady.n;
sim.pert.prun.n1 = zeros(T,1); sim.pert.prun.n1(1) = steady.n;
sim.pert.prun.n2 = zeros(T,1); sim.pert.prun.n2(1) = steady.n;
sim.pert.time    = zeros(T,1); sim.pert.time(1)    = steady.n;

%Simulation
for i = 2:T,
    tic
    i
    %Productivity
    zhat = lnz(i);
    
    %Projection
    nlag        = sim.proj(i-1);
    unow        = 1-nlag;
    gnow        = vpol(zhat,nlag);
    vnow        = unow*(par.phi0*gnow/par.cost)^(1/par.phi);
    sim.proj(i) = (1-par.rho_x)*nlag+par.phi0*unow^par.phi*vnow^(1-par.phi);
    
    %First-order
    nhat = sim.pert.norm.n1(i-1)-steady.n;
    sim.pert.norm.n1(i) = decision(:,2)'*[1;-1;nhat;0;zhat;0;0;0;0;0;0];    %[...; -1; ...] is real first-order perturbation solution.
                                                                            %[...;  0; ...] includes correction term and looks better!
    
    %Second-order
    nhat = sim.pert.norm.n2(i-1)-steady.n;
    sim.pert.norm.n2(i) = decision(:,2)'*[1;0;nhat;0;zhat;nhat^2;0;0;zhat^2;nhat*zhat;0];
    
    %Second-order with pruning
    nhat1               = sim.pert.prun.n1(i-1)-steady.n;
    sim.pert.prun.n1(i) = decision(:,2)'*[1;0;nhat1;0;zhat;0;0;0;0;0;0];
    nhat2               = sim.pert.prun.n2(i-1)-steady.n;
    sim.pert.prun.n2(i) = decision(:,2)'*[1;0;nhat2;0;zhat;nhat1^2;0;0;zhat^2;nhat1*zhat;0];
    
    %First-order alternative
    % the alterr (Aalterr) programs do perturbation plus without (with) the short-cut 
    % when using alterr or Aalterr you do 1-step ahead perturbation plus
    % when using alterr1 you do multi-step ahead perturbation plus
    % how many steps depend on what you specify in alterr1 (it can either
    % call alterr2 or close using the perturbation solution)
    znow = zhat; nlag = sim.pert.time(i-1);
    %sim.pert.time(i) = Aalterr(znow,nlag,par,steady,decision,z,p);
    sim.pert.time(i) = fsolve(@(nnow) alterr(nnow,znow,nlag,par,steady,decision,z,p),sim.pert.time(i-1),options);
    [i sim.proj(i) sim.pert.norm.n1(i) sim.pert.time(i)]
    toc
%pause
end

%Plotting
T1 = 1;
T2 = 100;
figure('Name','Simulation')
if iplotpert2 == 0
    plot(T1:T2,[sim.proj(T1:T2),sim.pert.norm.n1(T1:T2),sim.pert.time(T1:T2),sim.pert.prun.n2(T1:T2)])
    legend('Projection','First-order perturbation','First-order alternative','Pruning perturbation')
else
    plot(T1:T2,[sim.proj(T1:T2),sim.pert.norm.n1(T1:T2),sim.pert.norm.n2(T1:T2),sim.pert.time(T1:T2),sim.pert.prun.n2(T1:T2)])
    legend('Projection','First-order perturbation','Second-order perturbation','First-order alternative','Pruning perturbation')
end
title('Simulation')
box off


%% Accuracy

if iplotpert2 == 0
    
    captruth = sim.proj;
    cap1stAL = sim.pert.time;
    cap1stno = sim.pert.norm.n1;
    cap2ndp2 = sim.pert.prun.n2;

    error_max_cap_ours_1  =  max( min([abs((cap1stAL-captruth)/mean(captruth)),abs((cap1stAL-captruth)./captruth)],[],2) );
    error_ave_cap_ours_1  = mean( min([abs((cap1stAL-captruth)/mean(captruth)),abs((cap1stAL-captruth)./captruth)],[],2) );

    error_max_cap_pert_1  =  max( min([abs((cap1stno-captruth)/mean(captruth)),abs((cap1stno-captruth)./captruth)],[],2) );
    error_ave_cap_pert_1  = mean( min([abs((cap1stno-captruth)/mean(captruth)),abs((cap1stno-captruth)./captruth)],[],2) );

    error_max_cap_pert_2p =  max( min([abs((cap2ndp2-captruth)/mean(captruth)),abs((cap2ndp2-captruth)./captruth)],[],2) );
    error_ave_cap_pert_2p = mean( min([abs((cap2ndp2-captruth)/mean(captruth)),abs((cap2ndp2-captruth)./captruth)],[],2) );
   
    xx1 = sprintf('ours1  %8.5f %8.5f',[error_max_cap_ours_1  error_ave_cap_ours_1   ]);
    xx2 = sprintf('pert1  %8.5f %8.5f',[error_max_cap_pert_1  error_ave_cap_pert_1  ]);
    xx3 = sprintf('pert2p %8.5f %8.5f',[error_max_cap_pert_2p error_ave_cap_pert_2p ]);
    disp('ERRORS'),disp(xx1),disp(xx2),disp(xx3)
    disp('ave stdev minimum maximum')
    xx1=sprintf('truth  %8.5f %8.5f %8.5f %8.5f',[mean(captruth) std(captruth) min(captruth) max(captruth)]);
    xx2=sprintf('ours1  %8.5f %8.5f %8.5f %8.5f',[mean(cap1stAL) std(cap1stAL) min(cap1stAL) max(cap1stAL)]);
    xx3=sprintf('pert1  %8.5f %8.5f %8.5f %8.5f',[mean(cap1stno) std(cap1stno) min(cap1stno) max(cap1stno)]);
    xx4=sprintf('pert2p %8.5f %8.5f %8.5f %8.5f',[mean(cap2ndp2) std(cap2ndp2) min(cap2ndp2) max(cap2ndp2)]);
    disp(xx1),disp(xx2),disp(xx3),disp(xx4)
    YY = [captruth cap1stAL cap1stno cap2ndp2 ];

else

    captruth = sim.proj;
    cap1stAL = sim.pert.time;
    cap1stno = sim.pert.norm.n1;
    cap2ndno = sim.pert.norm.n2;
    cap2ndp2 = sim.pert.prun.n2;

    error_max_cap_ours_1  =  max( min([abs((cap1stAL-captruth)/mean(captruth)),abs((cap1stAL-captruth)./captruth)],[],2) );
    error_ave_cap_ours_1  = mean( min([abs((cap1stAL-captruth)/mean(captruth)),abs((cap1stAL-captruth)./captruth)],[],2) );

    error_max_cap_pert_1  =  max( min([abs((cap1stno-captruth)/mean(captruth)),abs((cap1stno-captruth)./captruth)],[],2) );
    error_ave_cap_pert_1  = mean( min([abs((cap1stno-captruth)/mean(captruth)),abs((cap1stno-captruth)./captruth)],[],2) );

    error_max_cap_pert_2  =  max( min([abs((cap2ndno-captruth)/mean(captruth)),abs((cap2ndno-captruth)./captruth)],[],2) );
    error_ave_cap_pert_2  = mean( min([abs((cap2ndno-captruth)/mean(captruth)),abs((cap2ndno-captruth)./captruth)],[],2) );

    error_max_cap_pert_2p =  max( min([abs((cap2ndp2-captruth)/mean(captruth)),abs((cap2ndp2-captruth)./captruth)],[],2) );
    error_ave_cap_pert_2p = mean( min([abs((cap2ndp2-captruth)/mean(captruth)),abs((cap2ndp2-captruth)./captruth)],[],2) );
   
    format short
    xx1 = sprintf('ours1  %8.5f %8.5f',[error_max_cap_ours_1  error_ave_cap_ours_1   ]);
    xx2 = sprintf('pert1  %8.5f %8.5f',[error_max_cap_pert_1  error_ave_cap_pert_1  ]);
    xx3 = sprintf('pert2  %8.5f %8.5f',[error_max_cap_pert_2  error_ave_cap_pert_2  ]);
    xx4 = sprintf('pert2p %8.5f %8.5f',[error_max_cap_pert_2p error_ave_cap_pert_2p ]);
    disp('ERRORS'),disp(xx1),disp(xx2),disp(xx3),disp(xx4)
    disp('ave stdev minimum maximum')
    xx1=sprintf('truth  %8.5f %8.5f %8.5f %8.5f',[mean(captruth) std(captruth) min(captruth) max(captruth)]);
    xx2=sprintf('ours1  %8.5f %8.5f %8.5f %8.5f',[mean(cap1stAL) std(cap1stAL) min(cap1stAL) max(cap1stAL)]);
    xx3=sprintf('pert1  %8.5f %8.5f %8.5f %8.5f',[mean(cap1stno) std(cap1stno) min(cap1stno) max(cap1stno)]);
    xx4=sprintf('pert2  %8.5f %8.5f %8.5f %8.5f',[mean(cap2ndno) std(cap2ndno) min(cap2ndno) max(cap2ndno)]);
    xx5=sprintf('pert2p %8.5f %8.5f %8.5f %8.5f',[mean(cap2ndp2) std(cap2ndp2) min(cap2ndp2) max(cap2ndp2)]);
    disp(xx1),disp(xx2),disp(xx3),disp(xx4),disp(xx5)
    YY = [captruth cap1stAL cap1stno cap2ndp2 ];


end


%Saving
print('-depsc2','-painters','Simulation')
fixPSlinestyle('Simulation.eps')
saveas(gcf,'Simulation','fig')


%Calculate relative standard deviations


if iplotpert2 == 0
    log_n_proj = sim.proj;
    log_n_proj_HP = log_n_proj-hpfilter2(log_n_proj,1600);
    [std(log_n_proj_HP)/std(lnz)]
else
    log_n_proj = sim.proj;
    log_n_pert = sim.pert.norm.n2;
    log_n_pert_HP = log_n_pert-hpfilter2(log_n_pert,1600);
    log_n_proj_HP = log_n_proj-hpfilter2(log_n_proj,1600);
    [std(log_n_proj_HP)/std(lnz) std(log_n_pert_HP)/std(lnz)]
end

